#include <iostream>
#include <sstream>
#include <fitsio.h>
#include <vector>
#include <string>

#include "pvo.hh"
#include "pvo-utils.hh"
#include "pvo-block.hh"
#include "pvo-header.hh"
#include "pvo-FITS.hh"

using namespace PVO;
using namespace PVO::Header;
using namespace PVO::Block;
using namespace PVO::FITS;


int main(int argc, char *argv[])
{
  if(argc>0){
    std::cerr << PVO::pvofixfilename_usage_text;
    return 0;
  }
  else{

    std::vector<std::string> namelist = PVO::getAllFitsFiles();
    std::vector<std::string> execlist,changelist;

    for(int i=0;i<namelist.size();i++){

      PSRFits fitsfile;
      fitsfile.setVerbosity(true);
      int status = fitsfile.openFile(namelist[i],READONLY);

      if(status==0){

	std::string obs_mode,hdrname="OBS_MODE";
	status = fitsfile.getKeyword<std::string>(hdrname, obs_mode);
	obs_mode = deblank(obs_mode);
      
	if(status==0){
	  if((obs_mode!="PSR")&&(obs_mode!="SEARCH"))
	    std::cout << "Warning: " << namelist[i]
		      << " has a non-standard OBS_MODE keyword: '"
		      << obs_mode << "'\n";
	  if(obs_mode=="PSR"){
	    std::stringstream systemcall,change;
	    std::string newname = namelist[i];

	    size_t dot = newname.rfind('.');
	    if(dot!=std::string::npos){
	      if(newname.substr(dot,newname.size())!=".rf"){
		newname.replace(dot, newname.size(),".rf");
		systemcall << "mv " << namelist[i] << " " << newname;
		execlist.push_back(systemcall.str());
		change << namelist[i] << " --> " << newname;
		changelist.push_back(change.str());
	      }
	    }
	    else{ // there is no extension in the filename, so add one.
	      systemcall << "mv " << namelist[i] << " " << namelist[i]<<".rf";
	      execlist.push_back(systemcall.str());
	      change << namelist[i] << " --> " << namelist[i]<<".rf";
	      changelist.push_back(change.str());
	    }
	  }
	}
	else std::cout << namelist[i] << " NO OBS_MODE KEYWORD!\n";

      }
      else {
	std::stringstream errmsg;
	errmsg << "Error opening file " << namelist[i] << "\n";
	PVOError("namechange" , errmsg.str() );
      }

      if(fitsfile.closeFile()!=0){
	std::stringstream errmsg;
	errmsg << "Error closing file " << namelist[i] << "\n";
	PVOError("namechange" , errmsg.str() );
      }

    }

    if(execlist.size()>0){
      std::cout << "\nA total of " << execlist.size()
		<<" files need to have their names changed. "
		<<"The changes will be:\n";
      for(int i=0;i<execlist.size();i++) std::cout << changelist[i] << "\n";
    
      char ans;
      std::cout << "\nProceed? (y/n) : " 
		<< std::flush;
      std::cin >> ans;
      if(ans=='y'){
	for(int i=0; i<execlist.size();i++) system(execlist[i].c_str());
      }  
    }
    else std::cout << "\nNo files needed to be changed.\n";

  }
}
